C   Linearized Barotropic Vorticity Equation Model
C
C     Initial value/response problem version
C
C     by Naoki Sato,  January 30, 2003

C ########################################
C      SETTING THE PARAMETERS
C ########################################

      PARAMETER (LX=144,MX=72,NNX=42)

      PARAMETER (IX=48*20,IO=48*1,DTM=0.1800E4)

      REAL DX(MX),DY(MX),F(LX,MX),
     +     BC(LX,MX+1,0:NNX,0:NNX),BS(LX,MX+1,0:NNX,0:NNX),W(LX,MX+1),
     +     GC(0:NNX,0:NNX),GS(0:NNX,0:NNX),
     +     PS0(LX,MX+1),U0(LX,MX),V0(LX,MX+1),
     +     PS1(LX,MX+1),U1(LX,MX),V1(LX,MX+1),
     +     XI0(LX,MX),XI1(LX,MX),DXI(LX,MX),
     +     XI (LX,MX),XIP(LX,MX),
     +     D1 (LX,MX),D2 (LX,MX+1)

      CHARACTER INFL*80,OUTFL*80,POFL1*80,POFL2*80,POFL3*80

C ########################################
C   1. PREPARATION
C ########################################

      INFL  = 'X3000.dat'
      OUTFL = 'X3001.dat'
      WRITE(6,'(1X,A21,I4)') 'Maximum wavenumber = ',NNX
      WRITE(6,'(1X,A14,A24)') 'Basic field = ',INFL
      WRITE(6,'(1X,A14,A24)') 'Output file = ',OUTFL
      WRITE(6,*) '===================================='

C   XI0 (LX,MX) : Relative vorticity of the basic field

C   Please specify the spherical harmonic function files and the weight
C   function file.
      WRITE(6,*)
     +  'Data file of spherical harmonic functions & weight function:'
      POFL1 = 'bases73_cos.dat'
      POFL2 = 'bases73_sin.dat'
      POFL3 = 'weight73.dat'
      WRITE(6,'(1X,A32,A24)') 'Harmonic functions (cos-type) = ',POFL1
      WRITE(6,'(1X,A32,A24)') 'Harmonic functions (sin-type) = ',POFL2
      WRITE(6,'(1X,A32,A24)') 'Weight function               = ',POFL3
      CALL POPRE (LX,MX+1,NNX,POFL1,POFL2,POFL3,BC,BS,W)
      WRITE(6,*) '===================================='

      CALL SETGR (LX,MX,DX,DY)
      CALL SETCR (LX,MX,F)
      CALL RDDAT (LX,MX,INFL,XI0)
      CALL DIAPS (LX,MX,NNX,BC,BS,W,XI0,PS0,GC,GS,D2)
      CALL DIAUV (LX,MX,DX,DY,PS0,U0,V0)

C ########################################
C   2. INITIAL CONDITIONS
C ########################################

C   XI1 (LX,MX) : Relative vorticity of the anomaly field

      CALL RESET (LX,MX,XI1)

C #### Subroutine for initial value problem ####
C   For initial value problem, remove the comment of the following line.
C      CALL INIT2 (LX,MX,20.,130.,10.,0.6E8,D1)

      CALL GCNV1 (LX,MX,D1,PS1)
      CALL DIAUV (LX,MX,DX,DY,PS1,U1,V1)
      CALL DIAXI (LX,MX,DX,DY,U1,V1,XI1)

      CALL SUMM  (LX,MX,XI0,XI1,XI)
      CALL SUMM  (LX,MX,XI0,XI1,XIP)

C #### Do loop 11 starts here ####

C   In roop 11, the time derivateve of vorticity is calculated,
C   and time-integration is performed for it.

      WRITE(6,*) 'Time-integration:'
      WRITE(6,'(1X,A22,1X,F9.3,1X,A4)')
     +                          'Time step            =',DTM,'sec.'
      WRITE(6,'(1X,A22,1X,I6)') 'Number of time steps =',IX
      WRITE(6,'(1X,A22,1X,I6)') 'Interval of output   =',IO
      WRITE(6,'(1X,A22,1X,I6)') 'Number of outputs    =',IX/IO
      WRITE(6,*) '===================================='
      WRITE(6,*) 'Time-integration started...'

      DO 11 I=1,IX

C ########################################
C   3. CALCULATION OF DEPENDENT VARIABLES
C ########################################

      CALL DIFF  (LX,MX,XI,XI0,XI1)
      CALL DIAPS (LX,MX,NNX,BC,BS,W,XI1,PS1,GC,GS,D2)
      CALL DIAUV (LX,MX,DX,DY,PS1,U1,V1)

C ########################################
C   4. DYNAMICAL PROCESSES
C ########################################

C   DXI (LX,MX) : Vorticity tendency

      CALL RESET (LX,MX,DXI)
      CALL ADVEC (LX,MX,DX,DY,U0,V0,XI1,DXI)
      CALL ADVEC (LX,MX,DX,DY,U1,V1,F  ,DXI)
      CALL ADVEC (LX,MX,DX,DY,U1,V1,XI0,DXI)
      CALL INTEL (LX,MX,DTM,DXI,XIP,XI)
      CALL POLES (LX,MX,XI)

C ########################################
C   5. PHYSICAL PROCESSES
C ########################################

      CALL DIFF  (LX,MX,XI,XI0,XI1)
      CALL RESET (LX,MX,DXI)
      CALL SDIFF (LX,MX,NNX,BC,BS,W,0.2E17,XI1,DXI,GC,GS,D1,D2)
C   For initial value problem, remove the comment of the following line.
C      CALL DMPNG (LX,MX,0.864E6,XI1,DXI)
C   For response problem, remove the comment of the following line.
      CALL DMPNG (LX,MX,0.432E6,XI1,DXI)

C #### Subroutine for response problem ####
C   For response problem, remove the comment of the following line.
      CALL FORCE2 (LX,MX,20.,130.,10.,1.0E-5,F,DXI)

      CALL INTEE (LX,MX,DTM,DXI,XIP,XI)
      CALL POLES (LX,MX,XI)

C ########################################
C   6. OUTPUT
C ########################################

      IF (MOD(I,IO).EQ.0) THEN
        CALL DIFF  (LX,MX,XI,XI0,XI1)
        WRITE(6,'(1X,I4,1X,A1,1X,I4)') I/IO,'/',IX/IO
        CALL WRDAT (LX,MX,OUTFL,I/IO,XI1)
      ENDIF

   11 CONTINUE

C #### Do loop 11 ends here ####

      STOP
      END

C ########################################
C   SUBROUTINES
C ########################################

C ========================================
C   INPUT/OUTPUT
C ========================================

      SUBROUTINE RDDAT (LX,MX,INFL,XI)
      REAL XI(LX,MX),XIR(144,73)
      CHARACTER INFL*80
      OPEN(10,FILE=INFL,
     +     STATUS='OLD',FORM='UNFORMATTED',
     +     ACCESS='DIRECT',RECL=4*(144*73))
      READ (10,REC=1) XIR
      CALL RDCNV1 (144,73,XIR)
      CALL RDCNV2 (144,73,XIR,XI)
      CLOSE(10)
      RETURN
      END

      SUBROUTINE RDCNV1 (LX,MX,G)
      REAL G(LX,MX)
      DO 11 M=1,MX/2
      M1 = M
      M2 = MX+1-M
      DO 12 L=1,LX
        G1 = G(L,M1)
        G2 = G(L,M2)
        G(L,M1) = G2
        G(L,M2) = G1
   12 CONTINUE
   11 CONTINUE
      RETURN
      END

      SUBROUTINE RDCNV2 (LX,MX,G,GG)
      REAL G(LX,MX),GG(LX,MX-1)
      DO 11 M=1,MX-1
      M1 = M
      M2 = M+1
      DO 12 L=1,LX
      L1 = L
      L2 = L+1
      IF (L2.GT.LX) L2 = 1
        GG(L,M) = G(L1,M1) + G(L2,M1)
     +          + G(L1,M2) + G(L2,M2)
        GG(L,M) = GG(L,M)/REAL(4)
   12 CONTINUE
   11 CONTINUE
      RETURN
      END

      SUBROUTINE WRDAT (LX,MX,OUTFL,I,XI)
      REAL XI(LX,MX),XIW(144,73)
      CHARACTER OUTFL*80
      CALL WRCNV2 (144,73,XI,XIW)
      CALL WRCNV1 (144,73,XIW)
      OPEN(11,FILE=OUTFL,
     +     STATUS='UNKNOWN',FORM='UNFORMATTED',
     +     ACCESS='DIRECT',RECL=4*(144*73))
      WRITE(11,REC=I) XIW
      CLOSE(11)
      RETURN
      END

      SUBROUTINE WRCNV1 (LX,MX,G)
      REAL G(LX,MX)
      DO 11 M=1,MX/2
      M1 = M
      M2 = MX+1-M
      DO 12 L=1,LX
        G1 = G(L,M1)
        G2 = G(L,M2)
        G(L,M1) = G2
        G(L,M2) = G1
   12 CONTINUE
   11 CONTINUE
      RETURN
      END

      SUBROUTINE WRCNV2 (LX,MX,G,GG)
      REAL G(LX,MX-1),GG(LX,MX)
      DO 11 M=1,MX
      M1=M
      M2=M+1
      DO 12 L=1,LX
      L1=L-1
      L2=L
      IF (L1.LT.1) L1=L1+LX
        GG(L,M) = 0.25E0*(G(L1,M1)+G(L2,M1)+G(L1,M2)+G(L2,M2))
   12 CONTINUE
   11 CONTINUE
      RETURN
      END

C ========================================
C   INITIAL ANOMALY/FORCING
C ========================================

      SUBROUTINE INIT1 (LX,MX,YC,XC,YD,XD,C,G)
C     -- The original by Simmons et al. (1983) --
      REAL G(LX,MX)
      CALL CONST (PI,RE,OM)
      CALL RESET (LX,MX,G)
      DO 11 M=1,MX
      Y = -REAL(90) + REAL(180)*(REAL(M)-0.5E0)/REAL(MX)
      DY = ABS(Y-YC)
      DO 12 L=1,LX
      X = REAL(360)*(REAL(L)-0.5E0)/REAL(LX)
      DX = AMOD(ABS(X-XC),360.)
      IF ((DY.LT.0.5E0*YD).AND.(DX.LT.0.5E0*XD)) THEN
        G(L,M) = C * (COS(PI*DX/XD)*COS(PI*DY/YD))**2
      ENDIF
   12 CONTINUE
   11 CONTINUE
      RETURN
      END

      SUBROUTINE INIT2 (LX,MX,B0,A0,R0,C,G)
C     -- Modified version --
      REAL G(LX,MX)
      CALL CONST (PI,RE,OM)
      CALL VECT  (PI/REAL(180)*A0,PI/REAL(180)*B0,X0,Y0,Z0)
      CALL RESET (LX,MX,G)
      DO 11 M=1,MX
      B = -REAL(90) + REAL(180)*(REAL(M)-0.5E0)/REAL(MX)
      DO 12 L=1,LX
      A = REAL(360)*(REAL(L)-0.5E0)/REAL(LX)
      CALL VECT (PI/REAL(180)*A,PI/REAL(180)*B,X,Y,Z)
      P = X0*X + Y0*Y + Z0*Z
      R = REAL(180)/PI * ACOS(P)
      IF (R.LT.R0) THEN
        G(L,M) = C * COS(0.5E0*PI*R/R0)**2
      ENDIF
C      WRITE(6,*) L,M,G(L,M)
C      IF (G(L,M).NE.0.0E0) STOP
   12 CONTINUE
   11 CONTINUE
      RETURN
      END

      SUBROUTINE FORCE1 (LX,MX,YC,XC,YD,XD,C,F,DG)
C     -- The original by Simmons et al. (1983) --
      REAL DG(LX,MX),F(LX,MX)
      CALL CONST (PI,RE,OM)
      DO 11 M=1,MX
      Y = -REAL(90) + REAL(180)*(REAL(M)-0.5E0)/REAL(MX)
      DY = ABS(Y-YC)
      DO 12 L=1,LX
      X = REAL(360)*(REAL(L)-0.5E0)/REAL(LX)
      DX = AMOD(ABS(X-XC),360.)
      IF ((DY.LT.0.5E0*YD).AND.(DX.LT.0.5E0*XD)) THEN
        D = C * (COS(PI*DX/XD)*COS(PI*DY/YD))**2
        DG(L,M) = DG(L,M) - F(L,M) * D
      ENDIF
   12 CONTINUE
   11 CONTINUE
      RETURN
      END

      SUBROUTINE FORCE2 (LX,MX,B0,A0,R0,C,F,DG)
C     -- Modified version --
      REAL DG(LX,MX),F(LX,MX)
      CALL CONST (PI,RE,OM)
      CALL VECT  (PI/REAL(180)*A0,PI/REAL(180)*B0,X0,Y0,Z0)
      DO 11 M=1,MX
      B = -REAL(90) + REAL(180)*(REAL(M)-0.5E0)/REAL(MX)
      DO 12 L=1,LX
      A = REAL(360)*(REAL(L)-0.5E0)/REAL(LX)
      CALL VECT (PI/REAL(180)*A,PI/REAL(180)*B,X,Y,Z)
      P = X0*X + Y0*Y + Z0*Z
      R = REAL(180)/PI * ACOS(P)
      IF (R.LT.R0) THEN
        D = C * COS(0.5E0*PI*R/R0)**2
        DG(L,M) = DG(L,M) - F(L,M) * D
      ENDIF
   12 CONTINUE
   11 CONTINUE
      RETURN
      END

      SUBROUTINE VECT (A,B,X,Y,Z)
      X = COS(B) * COS(A)
      Y = COS(B) * SIN(A)
      Z = SIN(B)
      RETURN
      END
